/*
 * GR-SAKURA USB HOST sample - MOUSE
 * Copyright (C) 2012 Yuuichi Akagawa
 *
 *******************************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 *******************************************************************************
 */
 
#include <rxduino.h>
#include <iodefine_gcc63n.h>
#include "tkusbhost.h"
#include "usb_defs.h"

int setIdle()
{
    unsigned short req = (USB_DIR_H2D|USB_REQ_TYPE_CLASS|USB_REQ_RECIPIENT_INTERFACE) | (HID_SET_IDLE << 8);
    return tkusbh_control_msg(req, 0, 0, 0, NULL, 1000);
}
     
int getReportDesc(unsigned char *buf, unsigned short len, byte ifnum)
{
    unsigned short req = (USB_DIR_D2H|USB_REQ_TYPE_STANDARD|USB_REQ_RECIPIENT_INTERFACE) | (HID_GET_DESCRIPTOR << 8);
    return tkusbh_control_msg(req, 0x2200, (unsigned short)ifnum, len, buf, 1000);
}

bool parseConfigDescr()
{
    char buf[128];
    unsigned char reportbuf[256];
    byte ifnum = 0;
    int i = 0;
    int len;
    bool isMouse = false;

    tkusbh_get_descriptor(CONFIG_DESCRIPTOR_TYPE, 0, buf, 128);
    len = USBHostInfo.ConfigDesc.wTotalLength;
    while(i < len)
    {
        if(buf[i+1] == CONFIG_DESCRIPTOR_TYPE){
            ConfigDesc_t *desc = (ConfigDesc_t *)&buf[i];
            i += buf[i+0];
            continue;
        }
        if(buf[i+1] == INTERFACE_DESCRIPTOR_TYPE){
            InterfaceDesc_t *desc = (InterfaceDesc_t *)&buf[i];
            ifnum = buf[i+2];
            i += buf[i+0];
            continue;
        }
        if(buf[i+1] == ENDPOINT_DESCRIPTOR_TYPE){
            EndpointDesc_t *desc = (EndpointDesc_t *)&buf[i];
            i += buf[i+0];
            continue;
        }
        if(buf[i+1] == USB_DESCRIPTOR_TYPE_HID)
        {
            HidDesc_t *desc = (HidDesc_t *)&buf[i];
            printf("HID Desc :%02X %02X %04X\n", desc->bLength, desc->bDescriptorType, desc->wDescriptorLength);
            getReportDesc(reportbuf, desc->wDescriptorLength, ifnum);
//   dump(reportbuf, (int)hiddescsize);
            if( reportbuf[0] == 0x05 && reportbuf[1] == 0x01 && reportbuf[2] == 0x09 && reportbuf[3] == 0x02){
                printf("Mouse\n");
                isMouse = true;
            }
            i += buf[i+0];
            continue;
        }else{
            i += buf[i+0];
            continue;
        }
    }
    return(isMouse);
}

bool usb_connected = false;
int blink=0;
bool isConnected()
{
    //Disconnect ?
    if( usb_connected == true && tkusbh_is_connected() == TKUSBH_DISCONNECT ) {
        tkusbh_disconnect(TIMEOUT_INFINITE);
        printf("Target disconnected.\n");
        usb_connected = false;
        digitalWrite(PIN_LED1, 0);
        digitalWrite(PIN_LED2, 0);
        digitalWrite(PIN_LED3, 0);
        return false;
    }
    if( usb_connected )
        return true;
    if(tkusbh_connect(500) != TKUSBH_OK)
    {
        printf("Waiting for connection...\n");
        digitalWrite(PIN_LED0, blink);
        blink = ((blink == 0) ? 1 : 0);
        return false;
    }
    if( parseConfigDescr() == false)
        return false;
    setIdle();
    usb_connected = true;
    digitalWrite(PIN_LED0, 1);
    tkusbh_start_interrupt_trans(1,4,INTERRUPT_IN,NULL);
    return true;
}

void setup()
{
    pinMode(PIN_LED0,OUTPUT);
    pinMode(PIN_LED1,OUTPUT);
    pinMode(PIN_LED2,OUTPUT);
    pinMode(PIN_LED3,OUTPUT);
    pinMode(PIN_SW  ,INPUT);

    //デバッグ出力はXBee
    Serial.begin(9600,SCI_SCI2B);
    Serial.setDefault();

    if(tkusbh_init() != TKUSBH_OK){
        printf("!!! USB Host Initialization ERROR !!! system halt\n");
        while(1);
    }
}

void loop()
{
    unsigned char buf[8];
    if( !isConnected() ) {
        delay(100);
        return;
    }

    int stat = tkusbh_interrupt_read(1,buf,8);
    printf("%02x %02x %02x %02x %02x\n", buf[0], buf[1], buf[2], buf[3], buf[4]);

    //left button
    if( buf[0] & 1 ){
        digitalWrite(PIN_LED3, 1);
    }else{
        digitalWrite(PIN_LED3, 0);
    }
    //right button
    if( buf[0] & 2 ){
        digitalWrite(PIN_LED2, 1);
    }else{
        digitalWrite(PIN_LED2, 0);
    }
    delay(5);
}

