/*
 * USB helper header
 * Copyright (C) 2012 Yuuichi Akagawa
 *
 *******************************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 *******************************************************************************
 */
#ifndef USB_DEFS_H
#define USB_DEFS_H
//bmRequestType
//DIRECTION
#define USB_DIR_H2D 0x00
#define USB_DIR_D2H 0x80
//TYPE
#define USB_REQ_TYPE_STANDARD 0x00
#define USB_REQ_TYPE_CLASS    0x20
#define USB_REQ_TYPE_VENDOR   0x40
//RECIEPIENT
#define USB_REQ_RECIPIENT_DEVICE    0x00
#define USB_REQ_RECIPIENT_INTERFACE 0x01
#define USB_REQ_RECIPIENT_ENDPOINT  0x02
#define USB_REQ_RECIPIENT_OTHER     0x03

//Descriptor types
#define USB_DESCRIPTOR_TYPE_DEVICE                     0x01
#define USB_DESCRIPTOR_TYPE_CONFIGURATION              0x02
#define USB_DESCRIPTOR_TYPE_STRING                     0x03
#define USB_DESCRIPTOR_TYPE_INTERFACE                  0x04
#define USB_DESCRIPTOR_TYPE_ENDPOINT                   0x05
#define USB_DESCRIPTOR_TYPE_DEVICE_QUALIFIER           0x06
#define USB_DESCRIPTOR_TYPE_OTHER_SPEED_CONFIGURATION  0x07
#define USB_DESCRIPTOR_TYPE_INTERFACE_POWER            0x08
#define USB_DESCRIPTOR_TYPE_HID                        0x21
#define USB_DESCRIPTOR_TYPE_HID_REPORT                 0x22

//Endpoint
//bmAtrributes
#define USB_ENDPOINT_TYPE_CTRL                         0x00
#define USB_ENDPOINT_TYPE_ISOC                         0x01
#define USB_ENDPOINT_TYPE_BULK                         0x02
#define USB_ENDPOINT_TYPE_INTR                         0x03
//Sync type
#define USB_ENDPOINT_SYNC_NONE                         0x00
#define USB_ENDPOINT_SYNC_ASYNC                        0x04
#define USB_ENDPOINT_SYNC_ADAPTIVE                     0x08
#define USB_ENDPOINT_SYNC_SYNC                         0x0C
//Usage type
#define USB_ENDPOINT_USAGE_DATA                        0x00
#define USB_ENDPOINT_USAGE_FEEDBACK                    0x10

#define USB_ENDPOINT_DIR_OUT                           0x00
#define USB_ENDPOINT_DIR_IN                            0x80

//Interface classes
#define USB_CLASS_AUDIO                                0x01
#define USB_CLASS_CDC_CONTROL                          0x02
#define USB_CLASS_HID                                  0x03
#define USB_CLASS_PHYSICAL                             0x05
#define USB_CLASS_PRINTER                              0x07
#define USB_CLASS_MSC                                  0x08
#define USB_CLASS_HUB                                  0x09
#define USB_CLASS_CDC_DATA                             0x0A
#define USB_CLASS_FIRMWARE_UPDATE                      0xFE
#define USB_CLASS_VENDOR                               0xFF

//Interface sub-classes
#define USB_SUBCLASS_MIDISTREAMING     0x03
#define USB_SUBCLASS_FIRMWARE_UPDATE   0x01
#define USB_SUBCLASS_IRDA              0x02

//HID
#define HID_BOOT_CODE                                  0x01    
#define HID_KEYBRD_BOOT_CODE                           0x01
#define HID_MOUSE_BOOT_CODE                            0x02


#define HID_GET_REPORT                                 0x01
#define HID_GET_IDLE                                   0x02
#define HID_GET_PROTOCOL                               0x03
#define HID_GET_DESCRIPTOR                             0x06
#define HID_SET_REPORT                                 0x09
#define HID_SET_IDLE                                   0x0a
#define HID_SET_PROTOCOL                               0x0b

typedef struct HidDesc_t
{
    unsigned char  bLength;
    unsigned char  bDescriptorType;
    unsigned short bcdHID;
    unsigned char  bCountryCode;
    unsigned char  bNumDescriptors;
    unsigned char  bHidDescriptorType;
    unsigned short wDescriptorLength;
} HidDesc_t;


#endif //USB_DEFS_H
